/*******************************************************************************
*
* A C T R O N  L I Z A R D  B O A R D  S U P P O R T  P A C K A G E
*
*                                                      Copyright (c) Actron GmbH
*                                                        written by Achim Dbler
*                                               and Maximilian Schwarz (DeeFuse)
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* Actron hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   The module LIZARD_ts is responsible to read the touch states from the
*   touch hardware via I2C. The touch driver supports multi-touch.
*
*******************************************************************************/

#include "LIZARD_ts.h"

// I2C variables from "LIZARD.c"
extern I2C_HandleTypeDef I2cHandle;
extern uint32_t i2c_timeout;

// Private types
typedef int (*cb_readInfo)();
typedef int (*cb_pollData)();
typedef uint8_t (*cb_updateTS_State)(TS_StateTypeDef* TS_State);

#pragma pack(1)

typedef struct
{
  uint8_t point_id:6;
  uint8_t tip:1;
  uint8_t res:1;
  uint16_t touchX;
  uint16_t touchY;
} ilitek_TouchPoint_t;

typedef struct
{
  uint8_t report_id;
  ilitek_TouchPoint_t finger[10];
  uint8_t reserved[10];
  uint8_t contactCount;
  uint8_t reserved2;
  uint8_t crc;
}ilitek_TouchData_t;

typedef struct
{
  uint8_t touchX_High:4;
  uint8_t reserved:2;
  uint8_t eventType:2;
  uint8_t touchX_Low;
  uint8_t touchY_High:4;
  uint8_t pointId:4;
  uint8_t touchY_Low;
  uint16_t touchZ;
}hycon_TouchPoint_t;

typedef struct
{
  uint8_t mode;
  uint8_t gesture;
  uint8_t contactCount:4;
  uint8_t reserved:4;
  hycon_TouchPoint_t finger[10]; // could be 11
}hycon_TouchData_t;

#pragma pack()

typedef struct
{
  uint16_t touchWidth;
  uint16_t touchHeight;
} IlitekDisplayInfo_t;

typedef struct
{
  uint8_t i2cAddress;
  cb_readInfo readInfo;
  cb_pollData pollData;
  cb_updateTS_State updateTouchState;
} TouchDriver_t;

// Private variables
static volatile int  IsTouchEvent    = 0;
static unsigned char TouchDataBuffer[ 64 ];
static unsigned int  ScaleWidth, ScaleHeight;
static int           LcdWidth, LcdHeight;
static TouchDriver_t *TouchDriver = NULL;

// Private function declarations
int hycon_readInfo();
int hycon_pollData();
uint8_t hycon_updateTouchState(TS_StateTypeDef* TS_State);

int ilitek_readInfo();
int ilitek_pollData();
uint8_t ilitek_updateTouchState(TS_StateTypeDef* TS_State);

static TouchDriver_t AvailableDrivers[] = {
  {
    .i2cAddress = 0x38,
    .readInfo = hycon_readInfo,
    .pollData = hycon_pollData,
    .updateTouchState = hycon_updateTouchState
  },
  {
    .i2cAddress = 0x41,
    .readInfo = ilitek_readInfo,
    .pollData = ilitek_pollData,
    .updateTouchState = ilitek_updateTouchState
  }
};

// Hycon specific functions
int hycon_readInfo()
{
  return HAL_OK;
}

int hycon_pollData()
{
  HAL_StatusTypeDef status;

  status = HAL_I2C_Mem_Read(&I2cHandle, (TouchDriver->i2cAddress<<1), 0x00, I2C_MEMADD_SIZE_8BIT, TouchDataBuffer, sizeof( TouchDataBuffer ), i2c_timeout);

  return status;
}

uint8_t hycon_updateTouchState(TS_StateTypeDef* TS_State)
{
  uint8_t noOfTouch = 0;

  TS_State->touchDetected = 0;

  hycon_TouchData_t* touchData = (hycon_TouchData_t*)TouchDataBuffer;

  for (int i = 0; i < NO_OF_FINGERS; i++ )
  {
    // check for touch down or touch contact
    if (( touchData->finger[i].eventType == 0b00 ) || ( touchData->finger[i].eventType == 0b10 ))
    {
      TS_State->touchX[ noOfTouch ] = (touchData->finger[i].touchX_High << 8) | touchData->finger[i].touchX_Low;
      TS_State->touchY[ noOfTouch ] = (touchData->finger[i].touchY_High << 8) | touchData->finger[i].touchY_Low;
      TS_State->touchId[ noOfTouch ] = touchData->finger[i].pointId;
      // EwPrint( "Touch ID %d at %d, %d\n", TS_State->touchId[ noOfTouch ], TS_State->touchX[ noOfTouch ], TS_State->touchY[ noOfTouch ]);
      noOfTouch++;
    }
  }

  TS_State->touchDetected = noOfTouch;

  return noOfTouch;
}

// Ilitek specific functions
int ilitek_readInfo()
{
  HAL_StatusTypeDef status;
  const uint8_t displayInfoCmd =  0x20 ;

  // Dummy read to get TouchIC working correctly
  HAL_I2C_Master_Receive(&I2cHandle, (TouchDriver->i2cAddress << 1), (uint8_t*)TouchDataBuffer, 1, 10);

  // Read Display info from TouchIC
  status = HAL_I2C_Mem_Read(&I2cHandle, (TouchDriver->i2cAddress << 1), displayInfoCmd, I2C_MEMADD_SIZE_8BIT, (uint8_t*)TouchDataBuffer, sizeof(TouchDataBuffer), i2c_timeout);

  IlitekDisplayInfo_t* dispInfo = (IlitekDisplayInfo_t*)TouchDataBuffer;

  // Calculate scalingfactors for for TouchCoordinate => ScreenCoordinate conversion
  ScaleWidth = dispInfo->touchWidth / LcdWidth;
  ScaleHeight = dispInfo->touchHeight / LcdHeight;

  return status;
}

int ilitek_pollData()
{
  HAL_StatusTypeDef status;

  status = HAL_I2C_Master_Receive(&I2cHandle, (TouchDriver->i2cAddress << 1), TouchDataBuffer, sizeof(TouchDataBuffer), i2c_timeout);

  return status;
}

uint8_t ilitek_updateTouchState(TS_StateTypeDef* TS_State)
{
  uint8_t noOfTouch = 0;

  TS_State->touchDetected = 0;

  ilitek_TouchData_t* touchData = (ilitek_TouchData_t*) &TouchDataBuffer;

  for (uint32_t i = 0; i < NO_OF_FINGERS; i++ )
  {
    // check for contact
    if (touchData->finger[i].tip)
    {
      TS_State->touchX[ noOfTouch ] = touchData->finger[i].touchX / ScaleWidth;
      TS_State->touchY[ noOfTouch ] = touchData->finger[i].touchY / ScaleHeight;
      TS_State->touchId[ noOfTouch ] = touchData->finger[i].point_id;
      // EwPrint( "Touch ID %d at %d, %d\n", TS_State->touchId[ noOfTouch ], TS_State->touchX[ noOfTouch ], TS_State->touchY[ noOfTouch ]);
      noOfTouch++;
    }
  }

  TS_State->touchDetected = noOfTouch;

  return noOfTouch;
}
/*******************************************************************************
* FUNCTION:
*   LIZARD_TS_Init
*
* DESCRIPTION:
*   Initialize the touch driver hardware.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void LIZARD_TS_Init( int screenWidth, int screenHeight )
{
  LcdWidth = screenWidth;
  LcdHeight = screenHeight;

  LIZARD_I2C_Init();

  GPIO_InitTypeDef GPIO_InitStructure;

  /* configure reset pin */
  __GPIOE_CLK_ENABLE();
  GPIO_InitStructure.Pin   = GPIO_PIN_4;
  GPIO_InitStructure.Mode  = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStructure.Pull  = GPIO_NOPULL;
  GPIO_InitStructure.Speed = GPIO_SPEED_FAST;
  HAL_GPIO_Init( GPIOE, &GPIO_InitStructure );

  /* configure interrupt pin and touch interrupt */
  GPIO_InitStructure.Pin   = GPIO_PIN_3;
  GPIO_InitStructure.Pull  = GPIO_PULLUP;
  GPIO_InitStructure.Speed = GPIO_SPEED_LOW;
  GPIO_InitStructure.Mode  = GPIO_MODE_IT_FALLING;
  HAL_GPIO_Init( GPIOE, &GPIO_InitStructure );
  HAL_NVIC_SetPriority((IRQn_Type)(EXTI3_IRQn), 15, 15 );
  HAL_NVIC_EnableIRQ((IRQn_Type)(EXTI3_IRQn));

  LIZARD_TS_Reset();

  HAL_StatusTypeDef status = HAL_ERROR;

  HAL_Delay(400); // ILITEK Page 7 T2

  // Itterate through all drivers to find a matching IC
  for(int i = 0; i < (sizeof(AvailableDrivers) / sizeof(TouchDriver_t)); i++)
  {
    status = HAL_I2C_IsDeviceReady(&I2cHandle, (AvailableDrivers[i].i2cAddress << 1), 3, 5);
    if( status == HAL_OK )
    {
      // Found TouchIC
      TouchDriver = &AvailableDrivers[i];
      TouchDriver->readInfo();
      break;
    }
  }
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_TS_GetState
*
* DESCRIPTION:
*   Read the current touch coordinates and touch ids from the touch data buffer
*   and return the current touch information in the provided data strucutre.
*
* ARGUMENTS:
*   Pointer to structure containing last touch information. In case of new
*   touch events, the structure will return current touch information.
*
* RETURN VALUE:
*   Returns the number of detected touch events, otherwise 0.
*
*******************************************************************************/
uint8_t LIZARD_TS_Get_State( TS_StateTypeDef* TS_State )
{
  uint8_t noOfTouch = 0;
  int touchStatus = HAL_ERROR;

  if(TouchDriver == NULL)
  {
    return 0;
  }


  if(IsTouchEvent)
  {
    touchStatus = TouchDriver->pollData();
    IsTouchEvent = 0;
    if( touchStatus != HAL_OK)
    {
      // Data poll invalid, retry next cycle
      return 0;
    }
  }

  /* access touch driver only in case of touch interrupt or in case that a
     touch cycle is already ongoing */
  if (touchStatus == HAL_OK || TS_State->touchDetected )
  {
    noOfTouch = TouchDriver->updateTouchState(TS_State);
  }

  return noOfTouch;
}


/*******************************************************************************
* FUNCTION:
*   LIZARD_TS_Reset
*
* DESCRIPTION:
*   Reset the touch driver.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void LIZARD_TS_Reset( void )
{
  GPIOE->ODR |= GPIO_PIN_4;
  HAL_Delay( 1 );
  GPIOE->ODR &= ~GPIO_PIN_4;
  HAL_Delay( 10 );
  GPIOE->ODR |= GPIO_PIN_4;
  HAL_Delay( 10 );
}


/*******************************************************************************
* FUNCTION:
*   EXTI3_IRQHandler
*
* DESCRIPTION:
*   Interrupt service routine to detect touch activities.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EXTI3_IRQHandler( void )
{
  if ( __HAL_GPIO_EXTI_GET_FLAG( GPIO_PIN_3 ))
  {
    /* set a flag to indicate that touch data is available */
    IsTouchEvent = 1;
    __HAL_GPIO_EXTI_CLEAR_IT( GPIO_PIN_3 );
  }
}

